@@ -23,6 +23,8 @@ logger = logging.getLogger(__name__)
 
 BALANCE_UPDATE_EVENT_NAME = "BalanceUpdate"
 TRUSTLINE_UPDATE_EVENT_NAME = "TrustlineUpdate"
+NETWORK_FREEZE_EVENT_NAME = "NetworkFreeze"
+NETWORK_UNFREEZE_EVENT_NAME = "NetworkUnfreeze"
 
 
 def topic_index_from_db(conn, addresses=None):
@@ -49,10 +51,8 @@ def get_logs(web3, addresses, fromBlock, toBlock):
 
 
 def get_events(web3, topic_index, fromBlock, toBlock) -> Iterable[logdecode.Event]:
-    return [
-        topic_index.decode_log(x)
-        for x in get_logs(web3, topic_index.addresses, fromBlock, toBlock)
-    ]
+    logs = get_logs(web3, topic_index.addresses, fromBlock, toBlock)
+    return topic_index.decode_logs(logs)
 
 
 def hexlify(d):
@@ -203,7 +203,15 @@ def delete_events(conn, fromBlock, addresses) -> List[Event]:
 
 def filter_events_for_graph(events):
     return [
-        event for event in events if event.name in ["TrustlineUpdate", "BalanceUpdate"]
+        event
+        for event in events
+        if event.name
+        in [
+            TRUSTLINE_UPDATE_EVENT_NAME,
+            BALANCE_UPDATE_EVENT_NAME,
+            NETWORK_FREEZE_EVENT_NAME,
+            NETWORK_UNFREEZE_EVENT_NAME,
+        ]
     ]
 
 
@@ -237,6 +245,7 @@ def null_replacing_graph_update(event: Event) -> GraphUpdate:
     if event.name == BALANCE_UPDATE_EVENT_NAME:
         null_event_args = copy.deepcopy(event.args)
         null_event_args["_value"] = 0
+        null_event_name = BALANCE_UPDATE_EVENT_NAME
     elif event.name == TRUSTLINE_UPDATE_EVENT_NAME:
         null_event_args = {
             "_creditor": event.args["_creditor"],
@@ -247,13 +256,22 @@ def null_replacing_graph_update(event: Event) -> GraphUpdate:
             "_interestRateReceived": 0,
             "_isFrozen": False,
         }
+        null_event_name = TRUSTLINE_UPDATE_EVENT_NAME
+    elif event.name == NETWORK_FREEZE_EVENT_NAME:
+        null_event_args = {}
+        # To replace a network freeze, we use a network unfreeze
+        null_event_name = NETWORK_UNFREEZE_EVENT_NAME
+    elif event.name == NETWORK_UNFREEZE_EVENT_NAME:
+        null_event_args = {}
+        # To replace a network unfreeze, we use a network freeze
+        null_event_name = NETWORK_FREEZE_EVENT_NAME
     else:
         raise RuntimeError(
             f"Tried to compute empty event of unexpected type {event.name}"
         )
 
     return GraphUpdate(
-        name=event.name,
+        name=null_event_name,
         args=null_event_args,
         address=event.address,
         timestamp=event.timestamp,
@@ -350,6 +368,27 @@ class Synchronizer:
         return graph_updates_to_feed
 
     def find_replacing_graph_update_for_missing(self, event: Event) -> GraphUpdate:
+        if event.name in [NETWORK_FREEZE_EVENT_NAME, NETWORK_UNFREEZE_EVENT_NAME]:
+            # No need to find the previous event in the database, just invert the freeze/unfreeze
+            return null_replacing_graph_update(event)
+
+        elif event.name in [BALANCE_UPDATE_EVENT_NAME, TRUSTLINE_UPDATE_EVENT_NAME]:
+            previous_graph_update = self.find_previous_trustline_graph_update(event)
+            if previous_graph_update is not None:
+                return previous_graph_update
+            return null_replacing_graph_update(event)
+
+        else:
+            raise RuntimeError(
+                f"Tried to find replacing event for event of unexpected type {event.name}"
+            )
+
+    def find_previous_trustline_graph_update(self, event):
+        assert event.name in [
+            BALANCE_UPDATE_EVENT_NAME,
+            TRUSTLINE_UPDATE_EVENT_NAME,
+        ], f"Tried to find previous event for event of unexpected type {event.name}"
+
         query_select = """SELECT transactionHash "transactionHash",
               address,
               eventName "event",
@@ -366,13 +405,9 @@ class Synchronizer:
         if event.name == BALANCE_UPDATE_EVENT_NAME:
             from_ = "_from"
             to = "_to"
-        elif event.name == TRUSTLINE_UPDATE_EVENT_NAME:
+        else:
             from_ = "_creditor"
             to = "_debtor"
-        else:
-            raise RuntimeError(
-                f"Tried to find previous event for event of unexpected type {event.name}"
-            )
 
         query = sql.SQL(
             query_select + "WHERE ((args->>{from_}=%s AND args->>{to}=%s) OR "
@@ -398,7 +433,7 @@ class Synchronizer:
                 if len(rows) == 1:
                     return build_graph_update_from_row(rows[0])
                 else:
-                    return null_replacing_graph_update(event)
+                    return None
 
     def feed_graph_updates(
         self, graph_feed_updates: Iterable[Union[Event, GraphUpdate]]
@@ -586,9 +621,11 @@ def importabi(addresses, contracts):
 def do_createtables(conn):
     with conn:
         with conn.cursor() as cur:
+            for table_name in ("events", "sync", "abis", "graphfeed"):
+                warn_if_table_exists(cur, table_name)
             cur.execute(
                 """
-                  CREATE TABLE events (
+                  CREATE TABLE IF NOT EXISTS events (
                     transactionHash TEXT NOT NULL,
                     blockNumber INTEGER NOT NULL,
                     address TEXT NOT NULL,
@@ -601,7 +638,7 @@ def do_createtables(conn):
                     PRIMARY KEY(transactionHash, address, blockHash, transactionIndex, logIndex)
                   );
 
-                  CREATE TABLE sync (
+                  CREATE TABLE IF NOT EXISTS sync (
                     syncid TEXT NOT NULL PRIMARY KEY,
                     last_block_number INTEGER NOT NULL,
                     addresses TEXT[] NOT NULL,
@@ -609,12 +646,12 @@ def do_createtables(conn):
                     latest_block_hash TEXT NOT NULL
                   );
 
-                  CREATE TABLE abis (
+                  CREATE TABLE IF NOT EXISTS abis (
                     contract_address TEXT NOT NULL PRIMARY KEY,
                     abi JSONB NOT NULL
                   );
 
-                  CREATE TABLE graphfeed (
+                  CREATE TABLE IF NOT EXISTS graphfeed (
                     address TEXT NOT NULL,
                     eventName TEXT NOT NULL,
                     args JSONB,
@@ -625,6 +662,20 @@ def do_createtables(conn):
             )
 
 
+def warn_if_table_exists(cur, table_name):
+    cur.execute(
+        """
+            SELECT to_regclass(%s);
+        """,
+        (table_name,),
+    )
+    table_exists = cur.fetchone()["to_regclass"] == table_name
+    if table_exists:
+        logger.warning(
+            f"Table {table_name} already exists, drop the tables first if you wish to recreate"
+        )
+
+
 @click.command()
 def createtables():
     logging.basicConfig(level=logging.INFO)
